==> epic4pre0.035.tar.gz: ircii-EPIC4pre0.035/source/output.c <==
/*
 * output.c: handles a variety of tasks dealing with the output from the irc
 * program 
 *
 * Written By Michael Sandrof
 *
 * Copyright(c) 1990 
 *
 * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
 */

#if 0
static	char	rcsid[] = "@(#)$Id: output.c,v 1.8 1994/07/02 02:32:13 mrg Exp $";
#endif

#include "irc.h"

#include <sys/stat.h>
#include <stdarg.h>

#include "output.h"
#include "vars.h"
#include "input.h"
#include "term.h"
#include "lastlog.h"
#include "window.h"
#include "screen.h"
#include "hook.h"
#include "ctcp.h"
#include "log.h"

	int	in_help = 0;

/* make this buffer *much* bigger than needed */
#define OBNOXIOUS_BUFFER_SIZE BIG_BUFFER_SIZE * 10
static	char	putbuf[OBNOXIOUS_BUFFER_SIZE + 1];

/* 
 * unflash: sends a ^[c to the screen
 * Must be defined to be useful, cause some vt100s really *do* reset when
 * sent this command. >;-)
 * Now that you can send ansi sequences, this is much less inportant.. 
 */
void unflash (void)
{
#ifdef HARD_UNFLASH
	fwrite("\033c", 2, 1, stdout);		/* hard reset */
#else
	fwrite("\033)0", 3, 1, stdout);		/* soft reset */
#endif
}

/* sig_refresh_screen: the signal-callable versino of refresh_screen */
RETSIGTYPE sig_refresh_screen (int unused)
{
	refresh_screen(0, NULL);
}

/*
 * refresh_screen: Whenever the REFRESH_SCREEN function is activated, this
 * swoops into effect 
 */
/* More keybinding lossage -- who the **** left all these old protos around? */
void refresh_screen (char dumb, char *dumber)
{
	unflash();
	term_clear_screen();
	if (term_resize())
		recalculate_windows();
	else
		redraw_all_windows();
	update_all_windows();
	update_input(UPDATE_ALL);
}

/* extern_write -- controls whether others may write to our terminal or not. */
/* This is basically stolen from bsd -- so its under the bsd copyright */
void extern_write (char *command, char *args, char *subargs)
{
	char *tty;
	struct stat sbuf;
	const int OTHER_WRITE = 020;

	if (!(tty = ttyname(2)))
	{
		yell("Could not figure out the name of your tty device!");
		return;
	}
	if (stat(tty, &sbuf) < 0)
	{
		yell("Could not get the information about your tty device!");
		return;
	}
	if (!args || !*args)
	{
		if (sbuf.st_mode & 020)
			say("Mesg is 'y'");
		else
			say("Mesg is 'n'");
		return;
	}
	switch (args[0])
	{
		case 'y' :
		{
			if (chmod(tty, sbuf.st_mode | OTHER_WRITE) < 0)
			{
				yell("Could not set your tty's mode!");
				return;
			}
			break;
		}
		case 'n' :
		{
			if (chmod(tty, sbuf.st_mode &~ OTHER_WRITE) < 0)
			{
				yell("Could not set your tty's mode!");
				return;
			}
			break;
		}
		default :
			say("Usage: /%s [Y|N]", command);
	}
}

/* init_windows:  */
void	init_screen (void)
{
	term_init();
	term_clear_screen();
	term_resize();
	new_window();
	recalculate_windows();
	update_all_windows();
	init_input();
	term_move_cursor(0, 0);
}

/*
 * put_it: the irc display routine.  Use this routine to display anything to
 * the main irc window.  It handles sending text to the display or stdout as
 * needed, add stuff to the lastlog and log file, etc.  Things NOT to do:
 * Dont send any text that contains \n, very unpredictable.  Tabs will also
 * screw things up.  The calling routing is responsible for not overwriting
 * the 1K buffer allocated.  
 *
 * For Ultrix machines, you can't call put_it() with floating point arguements.
 * It just doesn't work.  - phone, jan 1993.
 */
void put_it(const char *format, ...)
{
	if (window_display && format)
	{
		va_list args;
		va_start (args, format);
		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
		va_end(args);
		add_to_log(irclog_fp, putbuf);
		add_to_screen(putbuf);
	}
}

/* 
 * This is an alternative form of put_it which writes three asterisks
 * before actually putting things out.
 */
void say (const char *format, ...)
{
	if (window_display && format)
	{
		va_list args;

		strcpy(putbuf, get_string_var(BANNER_VAR));
		strcat(putbuf, " ");

		va_start (args, format);
		vsnprintf(putbuf + strlen(putbuf), OBNOXIOUS_BUFFER_SIZE - strlen(putbuf), format, args);
		va_end(args);

		add_to_log(irclog_fp, putbuf);
		add_to_screen(putbuf);
	}
}

void	yell(const char *format, ...)
{
	if (format)
	{
		va_list args;
		va_start (args, format);
		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
		va_end(args);
		add_to_log(irclog_fp, putbuf);
		add_to_screen(putbuf);
	}
}


/* help_put_it: works just like put_it, but is specially used by help */
void	help_put_it (const char *topic, const char *format, ...)
{
	if (format)
	{
		va_list args;
		va_start (args, format);
		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
		va_end(args);

		in_help = 1;
		if (do_hook(HELP_LIST, "%s %s", topic, putbuf))
		{
			/*
			 * XXXXX - setting this every time like this is absymal.
			 * Buts its needed to avoid major hacks to help.c, which
			 * ultimately makes sure everything goes to the right window.
			 */
			message_from((char *) 0, LOG_CURRENT);
			if (window_display)
			{
				add_to_log(irclog_fp, putbuf);
				add_to_screen(putbuf);
			}
			message_from((char *) 0, LOG_CRAP);
		}
		in_help = 0;
	}
}

==> epic4pre0.041.tar.gz: ircii-EPIC4pre0.041/source/output.c <==
/*
 * output.c: handles a variety of tasks dealing with the output from the irc
 * program 
 *
 * Written By Michael Sandrof
 *
 * Copyright(c) 1990 
 *
 * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
 */

#if 0
static	char	rcsid[] = "@(#)$Id: output.c,v 1.8 1994/07/02 02:32:13 mrg Exp $";
#endif

#include "irc.h"

#include <sys/stat.h>
#include <stdarg.h>

#include "output.h"
#include "vars.h"
#include "input.h"
#include "term.h"
#include "lastlog.h"
#include "window.h"
#include "screen.h"
#include "hook.h"
#include "ctcp.h"
#include "log.h"

	int	in_help = 0;

/* make this buffer *much* bigger than needed */
#define OBNOXIOUS_BUFFER_SIZE BIG_BUFFER_SIZE * 10
static	char	putbuf[OBNOXIOUS_BUFFER_SIZE + 1];

/* 
 * unflash: sends a ^[c to the screen
 * Must be defined to be useful, cause some vt100s really *do* reset when
 * sent this command. >;-)
 * Now that you can send ansi sequences, this is much less inportant.. 
 */
void unflash (void)
{
#ifdef HARD_UNFLASH
	fwrite("\033c", 2, 1, stdout);		/* hard reset */
#else
	fwrite("\033)0", 3, 1, stdout);		/* soft reset */
#endif
}

/* sig_refresh_screen: the signal-callable versino of refresh_screen */
SIGNAL_HANDLER(sig_refresh_screen)
{
	refresh_screen(0, NULL);
}

/*
 * refresh_screen: Whenever the REFRESH_SCREEN function is activated, this
 * swoops into effect 
 */
/* More keybinding lossage -- who the **** left all these old protos around? */
void refresh_screen (char dumb, char *dumber)
{
	unflash();
	term_clear_screen();
	if (term_resize())
		recalculate_windows();
	else
		redraw_all_windows();
	update_all_windows();
	update_input(UPDATE_ALL);
}

/* extern_write -- controls whether others may write to our terminal or not. */
/* This is basically stolen from bsd -- so its under the bsd copyright */
BUILT_IN_COMMAND(extern_write)
{
	char *tty;
	struct stat sbuf;
	const int OTHER_WRITE = 020;

	if (!(tty = ttyname(2)))
	{
		yell("Could not figure out the name of your tty device!");
		return;
	}
	if (stat(tty, &sbuf) < 0)
	{
		yell("Could not get the information about your tty device!");
		return;
	}
	if (!args || !*args)
	{
		if (sbuf.st_mode & 020)
			say("Mesg is 'y'");
		else
			say("Mesg is 'n'");
		return;
	}
	switch (args[0])
	{
		case 'y' :
		{
			if (chmod(tty, sbuf.st_mode | OTHER_WRITE) < 0)
			{
				yell("Could not set your tty's mode!");
				return;
			}
			break;
		}
		case 'n' :
		{
			if (chmod(tty, sbuf.st_mode &~ OTHER_WRITE) < 0)
			{
				yell("Could not set your tty's mode!");
				return;
			}
			break;
		}
		default :
			say("Usage: /%s [Y|N]", command);
	}
}

/* init_windows:  */
int	init_screen (void)
{
	if (term_init())
		return -1;

	term_clear_screen();
	term_resize();
	new_window();
	recalculate_windows();
	update_all_windows();
	init_input();
	term_move_cursor(0, 0);
	return 0;
}

/*
 * put_it: the irc display routine.  Use this routine to display anything to
 * the main irc window.  It handles sending text to the display or stdout as
 * needed, add stuff to the lastlog and log file, etc.  Things NOT to do:
 * Dont send any text that contains \n, very unpredictable.  Tabs will also
 * screw things up.  The calling routing is responsible for not overwriting
 * the 1K buffer allocated.  
 *
 * For Ultrix machines, you can't call put_it() with floating point arguements.
 * It just doesn't work.  - phone, jan 1993.
 */
void put_it(const char *format, ...)
{
	if (window_display && format)
	{
		va_list args;
		va_start (args, format);
		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
		va_end(args);
		add_to_log(irclog_fp, putbuf);
		add_to_screen(putbuf);
	}
}

/* 
 * This is an alternative form of put_it which writes three asterisks
 * before actually putting things out.
 */
void say (const char *format, ...)
{
	if (window_display && format)
	{
		va_list args;

		strcpy(putbuf, get_string_var(BANNER_VAR));
		strcat(putbuf, " ");

		va_start (args, format);
		vsnprintf(putbuf + strlen(putbuf), OBNOXIOUS_BUFFER_SIZE - strlen(putbuf), format, args);
		va_end(args);

		add_to_log(irclog_fp, putbuf);
		add_to_screen(putbuf);
	}
}

void	yell(const char *format, ...)
{
	if (format)
	{
		va_list args;
		va_start (args, format);
		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
		va_end(args);
		do_hook(YELL_LIST, "%s", putbuf);
		add_to_log(irclog_fp, putbuf);
		add_to_screen(putbuf);
	}
}


/* help_put_it: works just like put_it, but is specially used by help */
void	help_put_it (const char *topic, const char *format, ...)
{
	if (format)
	{
		va_list args;
		va_start (args, format);
		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
		va_end(args);

		in_help = 1;
		if (do_hook(HELP_LIST, "%s %s", topic, putbuf))
		{
			/*
			 * XXXXX - setting this every time like this is absymal.
			 * Buts its needed to avoid major hacks to help.c, which
			 * ultimately makes sure everything goes to the right window.
			 */
			message_from((char *) 0, LOG_CURRENT);
			if (window_display)
			{
				add_to_log(irclog_fp, putbuf);
				add_to_screen(putbuf);
			}
			message_from((char *) 0, LOG_CRAP);
		}
		in_help = 0;
	}
}
